import java.io.File;
import java.io.IOException;
import java.util.*;
import olif.*;
import oapi.*;
import support.csv.*;
import support.uuid.*;
import support.spellcheck.*;

public class OAPICLI {

	final static String sConfigFile		= "../conf/config.xml";
	final static String sInputXMLFile		= "input.xml";
	final static String sInputCSVFile		= "input.csv";
	final static String sOutputFile 		= "output.xml";
	final static String sTransformFile	= "transform.xsl";
	final static String sDictionaryFile	= "Quik.dic";
	
    public static boolean convertXMLToOlif() {
		File configFile = new File(sConfigFile);
		File transformFile  = new File(sTransformFile);
		File inputFile = new File(sInputXMLFile);
		File outputFile = new File(sOutputFile);
		boolean success = false;
		System.out.println();
		System.out.println("XML conversion started:");
		try {
			OAPITemplate oapiT = new OAPITemplate(configFile);
			OAPIXSLTemplate transT = new OAPIXSLTemplate(inputFile, transformFile);
   			OAPIConvert.convertXMLToOlif(oapiT, transT, inputFile, outputFile);
			success = true;
			System.out.println("success: XML conversion finished");
       	} catch (Exception e) {
            	System.out.println("Error converting XML to OLIF :" + e.getMessage());
        }
		return success;
	}

    public static boolean convertCSVToOlif() {
		File configFile = new File(sConfigFile);
		File inputFile = new File(sInputCSVFile);
		File outputFile = new File(sOutputFile);
		boolean success = false;
		System.out.println();
		System.out.println("CSV conversion started:");
		try {
			OAPITemplate oapiT = new OAPITemplate(configFile);
			OAPIConvert.convertCSVToOlif(oapiT, inputFile, outputFile);
			success = true;
			System.out.println("success: CSV conversion finished");
		} catch (Exception e) {
			System.out.println("Error converting CSV to OLIF :" + e.getMessage());
		}
		return success;
    }

	public static boolean spellcheck() {
		File configFile = new File(sConfigFile);
		File inputFile = new File(sOutputFile);
		File outputFile = new File(sOutputFile);
		File dictionaryFile = new File(sDictionaryFile);
		boolean success = false;
		List result= null;
		System.out.println();
		System.out.println("spellcheck started:");

		try {
			Olif olif = OAPICreate.createOlifObject(inputFile);
			String msg = "";
			Iterator it = OAPIModify.getEntryIterator(olif);
			Entry en = null;
			Entry newE = null;
			while (it.hasNext()) {
				en = (Entry)it.next();
				Enumeration em = en.enumerateTransfer();
				int t=0;
				while (em.hasMoreElements()) { t++; em.nextElement(); }
				if (t > 1) {
					java.util.List list = getList(en, dictionaryFile);
					System.out.println((en.getTransfer(1)).getTrTarget());
					if (list != null) {
						String header = "spelling error: "+ (en.getMono()).getKeyDC().getCanForm().getContent() +
"\n\n";
						Iterator i = list.iterator();
						while (i.hasNext()) {
							msg += ( ((Word)i.next()).getWord() ) + "\n";
						}
						if (!(msg.equals(""))) {
							String ex = ((Word)list.get(0)).getWord();
							newE = OAPICreate.createEntryTemplate(OAPISerialize.readConfig(configFile), "new");
							(((newE.getMono()).getKeyDC()).getCanForm()).setContent(ex);
							newE.addTransfer(en.getTransfer(1));						
							newE = OAPIModify.enhanceEntry(newE, "cn", "/entry/mono/keyDC/semReading", "CORRECTED");
							msg = msg + "\n exchanging with " + ex +"\n";
							olif = OAPIModify.exchangeOlifEntry(olif, en, newE);
					   		System.out.println(msg);
						}
					}
					msg = "";
				}
			}
			OAPISerialize.writeOlif(olif, outputFile);
   			success = true;
		} catch (Exception e) {
        	System.out.println("Error during spellcheck :" + e.getMessage());
		}
		return success;
	}
	
	private static List getList(Entry en, File dictionaryFile) {
		List result = null;
		try {
			result = OAPIModify.spellcheck(en, dictionaryFile);
		} catch (Exception e) {
        	System.out.println("Error during spellcheck :" + e.getMessage());
		}
    	return result;  
	}

	public static boolean validate() {
		File inputFile = new File(sOutputFile);
		boolean success = false;
		System.out.println();
		System.out.println("validation started:");
		try {
			Olif o = OAPICreate.createOlifObject(inputFile);
			success = true;
			System.out.println("success: Olif document is valid");
        } catch (Exception e) {
            System.out.println("Error during validation :" + e.getMessage());
        }
        return success;
	}

	public static boolean enhance(String path) {
		File configFile = new File(sConfigFile);
		File inputFile = new File(sOutputFile);
		File outputFile = new File(sOutputFile);
		boolean success = false;
		System.out.println();
		System.out.println("enhancing element started:");
		try {
			Olif olif = OAPICreate.createOlifObject(inputFile);
			Entry e = (olif.getBody()).getEntry(0);
			Entry newE = OAPICreate.createEntryTemplate(OAPISerialize.readConfig(configFile), "new");
			newE = OAPIModify.enhanceEntry(newE, "cn", path, "ENHANCED");
			olif = OAPIModify.exchangeOlifEntry(olif, e, newE);
			OAPISerialize.writeOlif(olif, outputFile);
			success = true;
			System.out.println("success: enhancing element finished");
        } catch (Exception e) {
            System.out.println("Error enhancing element :" + e.getMessage());
        }
        return success;
	}
    		
	public static boolean lookup(String term) {
    	File inputFile = new File(sOutputFile);
		System.out.println();
		System.out.println("lookup started:");
		boolean success = false;
		try {
			Iterator it = OAPIModify.getEntryIterator(OAPICreate.createOlifObject(inputFile));
			String SID = null;
			while (it.hasNext()) {
				Entry inner = (Entry)it.next();
				if ( ((inner).getMono().getKeyDC().getCanForm().getContent()).equals(term) ) {
					SID = ((inner.getTransfer(1)).getTrTarget());
				}
			}
			it = OAPIModify.getEntryIterator(OAPICreate.createOlifObject(inputFile));
			while (it.hasNext()) {
				Entry inner = (Entry)it.next();
				if ( (inner).getConceptUniversalId().equals(SID) ) {
					System.out.println((inner).getMono().getKeyDC().getCanForm().getContent());
				}
			}
			success = true;
			System.out.println("success: lookup successfully finished");
		} catch (Exception e) {
            System.out.println("Error enhancing element :" + e.getMessage());
		}
		return success;
  	}
		
	public static void main(String[] args) {
		boolean result = false;
		if (args[0].equals("-v")) {
			result = validate();
		} else {
		if (args[0].equals("-csv")) {
			result = convertCSVToOlif();
		} else {
		if (args[0].equals("-xml")) {
			result = convertXMLToOlif();
		} else {
		if (args[0].equals("-sc")) {
			result = spellcheck();
		} else {
		if (args[0].equals("-lu")) {
			result = lookup(args[1]);
		} else {
		if (args[0].equals("-e")) {
			result = enhance(args[1]);
		}}}}}}
		if (!result) {
			System.out.println("An error occurred.");
		}
	}
}
