package oapi;

/**
 * The base exception class for the OAPI framework
**/
public class OAPIException extends Exception {
    
    
    /**
     * The error code for this Exception
    **/
    private int _errorCode  = -1;
    
    /**
     * The message for this Exception
    **/
    private String    _message    = null;
    
    //----------------/
    //- Constructors -/
    //----------------/
    
    /**
     * Creates a new OAPIException with no message,
     * or nested Exception
    **/
    public OAPIException() {
        super();
    }
    
    /**
     * Creates a new OAPIException with the given message.
     * @param message the message for this Exception
    **/
    public OAPIException(String message) {
        super(message);
        this._message = message;
    }
    
    /**
     * Creates a new OAPIException with the given nested
     * exception.
     * @param errorCode the error code for this Exception
    **/
    public OAPIException(int errorCode) {
        super();
        this._errorCode = errorCode;
    }

    /**
     * Creates a new OAPIException with the given message
     * nested exception.
     * @param message the detail message for this exception
     * @param errorCode the error code for this Exception
    **/
    public OAPIException(String message, int errorCode) {
        super(message);
        this._message = message;
        this._errorCode = errorCode;
    }
    
    //------------------/
    //- Public Methods -/
    //------------------/
    
    /**
     * Returns the error code for this Exception, or -1 if no
     * error code exists.
     * @return the error code for this Exception, or -1 if
     * no error code exists
    **/
    public int getErrorCode() {
        return _errorCode;
    }
    
    /**
     * Returns the detail message for this Exception
     * @return the detail message for this Exception
    **/
    public String getMessage() {
        //-- simply return message, or if null, 
        //-- to prevent null pointer exceptions while printing
        //-- error message, return ""
        if (_message == null) return "";
        else return _message;
    }
    
    
    /**
     * Sets the error code for this Exception
     * @param errorCode the error code 
    **/
    public void setErrorCode(int errorCode) {
        this._errorCode = errorCode;
    }
    
    /**
     * Sets the message for this Exception
     * @param message the message for this Exception
    **/
    public void setMessage(String message) {
        this._message = message;
    }
    
    
}
