package oapi;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.PrintStream;

/**
 * This exception can be used to signal serialize exceptions
**/
public class OAPISerializeException extends OAPIException {
    
    //------------------/
    //- error messages -/
    //------------------/
    
    /**
     * A nested exception
    **/
    private Exception _exception  = null;
    
    /**
     * Creates a new OAPISerializeException with no message,
     * or nested Exception
    **/
    public OAPISerializeException() {
        super();
    }
    
    /**
     * Creates a new OAPISerializeException with the given message.
     * @param message the message for this Exception
    **/
    public OAPISerializeException(String message) {
        super(message);
    }

    /**
     * Creates a new OAPISerializeException with the given message.
     * @param message the message for this Exception
     * @param errorCode the errorCode for this Exception
    **/
    public OAPISerializeException(String message, int errorCode) {
        super(message, errorCode);
    }
    
    /**
     * Creates a new OAPISerializeException with the given nested
     * exception.
     * @param exception the nested exception
    **/
    public OAPISerializeException(Exception exception) {
        super();
        if (exception != null) {
            super.setMessage(exception.getMessage());
            this._exception = exception;
        }
    }

    /**
     * Creates a new OAPISerializeException with the given message
     * and nested exception.
     * @param message the detail message for this exception
     * @param exception the nested exception
    **/
    public OAPISerializeException(String message, Exception exception) {
        super(message);
        this._exception = exception;
    }

    /**
     * Creates a new OAPISerializeException with the given message,
     * nested exception, and errorCode.
     * @param message the detail message for this exception
     * @param exception the nested exception
     * @param errorCode the errorCode for this Exception
    **/
    public OAPISerializeException
        (String message, Exception exception, int errorCode) 
    {
        super(message, errorCode);
        this._exception = exception;
    }
    
    /**
     * Returns the exception, which in turn caused this Exception to
     * be thrown, or null if nested exception exists.
     * @return the exception, which in turn caused this Exception to
     * be thrown, or null if nested exception exists.
    **/
    public Exception getException() {
        return _exception;
    }
    
    /**
     * Returns the String representation of this Exception
     * @return the String representation of this Exception
    **/
    public String toString() {
		return getMessage();
    }



    public void printStackTrace()
    {
        if ( _exception == null )
            super.printStackTrace();
        else
            _exception.printStackTrace();
    }

    public void printStackTrace( PrintWriter printer )
    {
        if ( _exception == null )
            super.printStackTrace( printer );
        else
            _exception.printStackTrace( printer);
    }

    public void printStackTrace( PrintStream printer )
    {
        if ( _exception == null )
            super.printStackTrace( printer );
        else
            _exception.printStackTrace( printer );
    }
    
}
