package oapi;

import java.io.File;
import org.w3c.dom.Document;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import java.util.Enumeration; 
import support.uuid.UUID;

import required.*;
import olif.*;

import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;

/**
* The OAPIXSLTemplate class provides dynamic functions and methods for repetitional tasks like conversions on XML files.
* A typical invokation looks like this:
* <p><code>
* import olif;<br>
* import oapi;<br>
* File configFile = new File("config.xml");<br>
* File transFile = new File("transform.xsl");<br>
* File inputFile = new File("input.xml");<br>
* File outputFile = new File("output.xml");<br>
* OAPITemplate oapiT = new OAPITemplate(configFile);<br>
* OAPIXSLTemplate transT = new OAPIXSLTemplate(inputFile, transFile);<br>
* OAPIConvert.convertXMLToOlif(oapiT, transT, inputFile, outputFile);
* </code></p><p>
* which uses the OAPITemplate and OAPIXSLTemplate classes to convert the XML file and therefore reads the config file and transforms the XML file only once.
* </p>
*/
public class OAPIXSLTemplate {

	Transformer transformer;
	TransformerFactory tFactory;
	DOMResult dr;
	DocumentBuilderFactory factory;
   	DocumentBuilder builder;
	
/**
* <p> This creates the OAPIXSLTemplate object and makes all arrangements. </p>
*
* @param inputFile XML file with arbitrary format 
* @param transFile XSL file with transform rules
* @throws OAPIException
*/
	public OAPIXSLTemplate(File inputFile, File transFile) throws Exception {
		factory = DocumentBuilderFactory.newInstance();
       	builder = factory.newDocumentBuilder();
       	Document d = builder.newDocument();
       	dr = new DOMResult(d);
		tFactory = TransformerFactory.newInstance();
		transformer = tFactory.newTransformer(new StreamSource(transFile));
		transformer.transform(new StreamSource(inputFile), dr);
	}
	
/**
* <p> This returns the OAPIXSLTemplate node object. </p>
*
* @return Node template node object
*/
	public Node getXSLTemplateNode() throws OAPIException {
		return this.dr.getNode();
	}

}