package support.spellcheck;

import java.io.*;
import java.util.*;
import com.crionics.util.*;

public class SpellDictionary {

    private FastHashMap h = new FastHashMap(65536);
    private DoubleMeta dm = new DoubleMeta();

    public String getCode(String word) {
        return dm.process(word);
    }

    public void createDictionary(File dict) {
        Date start = new Date();
        try {
            BufferedReader in = new BufferedReader(new FileReader(dict));
            String line = "";
            while (line != null) {
                line = in.readLine();
                if (line != null) {
                    put(line);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        Date end = new Date();

    }

    public void put(String word) {
        String code = dm.process(word);
        LinkedList list = (LinkedList) h.get(code);
        if (list != null) {
            list.add(word);
        } else {
            list = new LinkedList();
            list.add(word);
            h.put(code, list);
        }
    }

    public LinkedList get(String code) {
        LinkedList result = (LinkedList) h.get(code);
        if (result == null)
            result = new LinkedList();
        return result;
    }

    public boolean isCorrect(String word) {
        LinkedList possible = get(dm.process(word));
        if (possible.contains(word)) return true;
        return false;
    }

    public LinkedList getSuggestions(String word) {
        int threshold = 120;
        LinkedList nearmiss = new LinkedList();
        try {
            String code = dm.process(word);
            HashSet similars = new HashSet();
            similars.addAll(get(code));
            EditDistance distance = new EditDistance();
            // do some tranformations to pick up more results
            char[] replacelist = {'A','B','X', 'S', 'K', 'J', 'T', 'F', 'H', 'L', 'M', 'N', 'P', 'R', '0' };

            //interchange
            char[] charArray = code.toCharArray();
            for (int i = 0; i < code.length() - 1; i++) {
                char a = charArray[i];
                char b = charArray[i + 1];
                charArray[i] = b;
                charArray[i + 1] = a;
                similars.addAll(get(new String(charArray)));
                charArray[i] = a;
                charArray[i + 1] = b;
            }
            //change
            charArray = code.toCharArray();
            for (int i = 0; i < code.length(); i++) {
                char original = charArray[i];
                for (int j = 0; j < replacelist.length; j++) {
                    charArray[i] = replacelist[j];
                    similars.addAll(get(new String(charArray)));
                }
                charArray[i] = original;
            }
            //add
            charArray = (code += " ").toCharArray();
            int iy = charArray.length - 1;
            while (true) {
                for (int j = 0; j < replacelist.length; j++) {
                    charArray[iy] = replacelist[j];
                    similars.addAll(get(new String(charArray)));
                }
                if (iy == 0) break;
                charArray[iy] = charArray[iy - 1];
                --iy;
            }

            //delete
            code = code.trim();
            charArray = code.toCharArray();
            char[] charArray2 = new char[charArray.length - 1];
            for (int ix = 0; ix < charArray2.length; ix++) {
                charArray2[ix] = charArray[ix];
            }
            char a,b;
            a = charArray[charArray.length - 1];
            int ii = charArray2.length;
            while (true) {
                similars.addAll(get(new String(charArray2)));
                if (ii == 0) break;
                b = a;
                a = charArray2[ii - 1];
                charArray2[ii - 1] = b;
                --ii;
            }
            // rank results
            Word w = new Word();
            Iterator i = similars.iterator();
            while (i.hasNext()) {
                String s = (String) i.next();
                w = new Word();
                w.setWord(s);
                int d = distance.getDistance(word, s, new EditDistanceWeights());
                if (d <= threshold) {
                    w.setScore(d);
                    nearmiss.add(w);
                }
            }
            Collections.sort(nearmiss, w);
        } catch (Exception e) {
        }
        return nearmiss;
    }


}


